from os.path import join, dirname
import logging.config
from typing import TextIO

import click
import yaml
import jsonschema

from .registrar import register_file  # , deregister_file, deregister_identifier
from .daemon import run_daemon
from .config import load_config


def setup_logging(debug=False):
    logging.config.dictConfig(
        {
            "version": 1,
            "disable_existing_loggers": False,
            "formatters": {"brief": {"format": "%(levelname)s %(name)s: %(message)s"}},
            "handlers": {
                "console": {
                    "class": "logging.StreamHandler",
                    "level": "DEBUG" if debug else "INFO",
                    "formatter": "brief",
                }
            },
            "root": {
                "handlers": ["console"],
                "level": "DEBUG" if debug else "INFO",
            },
        }
    )


def validate_config(config):
    with open(join(dirname(__file__), "config-schema.yaml")) as f:
        schema = yaml.load(f)

    jsonschema.validate(config, schema)


@click.group()
def cli():
    pass


@cli.command(help="Run the registrar daemon, attaching to a Redis queue")
@click.option("--config-file", required=True, type=click.File("r"))
@click.option("--host", required=True, type=str)
@click.option("--port", required=True, type=int)
@click.option("--listen-queue", required=True, type=str)
@click.option("--progress-set", required=True, type=str)
@click.option("--failure-set", required=True, type=str)
@click.option("--success-set", required=True, type=str)
@click.option("--validate/--no-validate")
@click.option("--replace/--no-replace")
@click.option("--deregister-queue", type=str)
@click.option("--debug/--no-debug")
def daemon(
    config_file: TextIO,
    host: str,
    port: int,
    listen_queue: str,
    progress_set: str,
    failure_set: str,
    success_set: str,
    validate: bool = False,
    replace: bool = False,
    deregister_queue: str = None,
    debug: bool = False,
):
    setup_logging(debug)
    config = load_config(config_file)
    if validate:
        validate_config(config)
    run_daemon(
        config,
        replace,
        host,
        port,
        listen_queue,
        progress_set,
        failure_set,
        success_set,
        deregister_queue,
    )


@cli.command(help="Run a single, one-off registration")
@click.argument("item", type=str)
@click.option("--config-file", required=True, type=click.File("r"))
@click.option("--validate/--no-validate")
@click.option("--replace/--no-replace")
@click.option("--debug/--no-debug")
def register(
    item,
    config_file: TextIO,
    validate: bool = False,
    replace: bool = False,
    debug: bool = False,
):
    setup_logging(debug)
    config = load_config(config_file)
    if validate:
        validate_config(config)

    register_file(config, item, replace)


# @cli.command(help="Run a single, one-off de-registration")
# @click.argument("--path", type=str)
# @click.argument("--identifier", type=str)
# @click.option("--config-file", type=click.File("r"))
# @click.option("--validate/--no-validate")
# @click.option("--debug/--no-debug")
# def deregister(
#     file_path: str = None,
#     identifier: str = None,
#     config_file: str = None,
#     validate: bool = False,
#     debug: bool = False,
# ):
#     setup_logging(debug)
#     config = load_config(config_file)
#     if validate:
#         validate_config(config)

#     if file_path:
#         deregister_file(config, file_path)
#     elif identifier:
#         deregister_identifier(config, identifier)


if __name__ == "__main__":
    cli()
